#
# DecoMecha - diving decompression library.
#
# Copyright (C) 2013 by Artur Wroblewski <wrobell@pld-linux.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
The libdeco-ostc project profile unit tests for open circuit.
"""

from decomecha import DecoEngine

import unittest

class ProfileOCTestCase(unittest.TestCase):
    """
    Profile unit tests for open circuit.
    """
    def test_eh(self):
        """
        Test Empire Heritage OC dive

        MultiDeco::

            Depth Stop Run Mix pO2 EAD
            Des 68 - 3 18/45 - -
            Lvl 68 16 20 18/45 1.39 32
            Asc 36 - 23 18/45 - -
            Stp 36 0:48 24 18/45 0.82 15
            Stp 33 1:00 25 18/45 0.77 13
            Stp 30 1:00 26 18/45 0.72 11
            Stp 27 1:00 27 18/45 0.66 10
            Stp 24 2:00 29 18/45 0.61 8
            Stp 21 2:00 31 50 1.54 9
            Stp 18 2:00 33 50 1.39 7
            Stp 15 2:00 35 50 1.24 5
            Stp 12 4:00 39 50 1.10 3
            Stp 9 6:00 45 50 0.95 1
            Stp 6 7:00 52 100 1.60 0
            Stp 3 13 65 100 1.30 0
            Sfc - - 65 100 - -

            Dive# 1,  ZHLB + GF 30/85
            Elevation = 0 m
            CNS = 41%
            OTU's = 88

            Gas  18/45 = 3754 ltr.
            Gas  50 = 732 ltr.
            Gas  100 = 558 ltr.

        gfdeco::

                                      DECOMPRESSION CALCULATION PROGRAM
                                    Developed in FORTRAN by Erik C. Baker
             
             
             
            Description:    dive to 68m for 20min on TX18/45, deco: ean50, o2                    
             
            Gasmix Summary:                        FO2    FHe    FN2
                                      Gasmix # 1  0.180  0.450  0.370
                                      Gasmix # 2  0.500  0.000  0.500
                                      Gasmix # 3  1.000  0.000  0.000
             
             
                                                DIVE PROFILE
             
            Seg-  Segm.  Run   | Gasmix | Ascent    From     To      Rate    | Constant
            ment  Time   Time  |  Used  |   or     Depth   Depth    +Dn/-Up  |  Depth
              #   (min)  (min) |    #   | Descent  (mswg)  (mswg)  (msw/min) |  (mswg)
            ----- -----  ----- | ------ | -------  ------  ------  --------- | --------
              1     3.4    3.4 |    1   | Descent     0.     68.      20.0   |
              2    16.6   20.0 |    1   |                                    |    68.
             
             
                                           DECOMPRESSION PROFILE
             
                      Leading compartment enters the decompression zone at   49.2 mswg
                             Deepest possible decompression stop is   48.0 mswg
             
            Seg-  Segm.  Run   | Gasmix | Ascent   Ascent         |  DECO   STOP   RUN
            ment  Time   Time  |  Used  |   To      Rate    Grad. |  STOP   TIME   TIME
              #   (min)  (min) |    #   | (mswg) (msw/min) Factor | (mswg)  (min)  (min)
            ----- -----  ----- | ------ | ------ --------- ------ | ------  -----  -----
              3     2.4   22.4 |    1   |   36.    -10.0          |
              4     0.6   23.0 |    1   |                   0.30  |    36      1     23
              5     0.3   23.3 |    1   |   33.    -10.0          |
              6     0.7   24.0 |    1   |                   0.35  |    33      1     24
              7     0.3   24.3 |    1   |   30.    -10.0          |
              8     0.7   25.0 |    1   |                   0.39  |    30      1     25
              9     0.3   25.3 |    1   |   27.    -10.0          |
             10     0.7   26.0 |    1   |                   0.44  |    27      1     26
             11     0.3   26.3 |    1   |   24.    -10.0          |
             12     1.7   28.0 |    1   |                   0.48  |    24      2     28
             13     0.3   28.3 |    1   |   21.    -10.0          |
             14     1.7   30.0 |    2   |                   0.53  |    21      2     30
             15     0.3   30.3 |    2   |   18.    -10.0          |
             16     0.7   31.0 |    2   |                   0.58  |    18      1     31
             17     0.3   31.3 |    2   |   15.    -10.0          |
             18     2.7   34.0 |    2   |                   0.62  |    15      3     34
             19     0.3   34.3 |    2   |   12.    -10.0          |
             20     2.7   37.0 |    2   |                   0.67  |    12      3     37
             21     0.3   37.3 |    2   |    9.    -10.0          |
             22     4.7   42.0 |    2   |                   0.71  |     9      5     42
             23     0.3   42.3 |    2   |    6.    -10.0          |
             24     6.7   49.0 |    3   |                   0.76  |     6      7     49
             25     0.3   49.3 |    3   |    3.    -10.0          |
             26    12.7   62.0 |    3   |                   0.80  |     3     13     62
             27     0.3   62.3 |    3   |    0.    -10.0          |
                                                            0.85  |


        """
        de = DecoEngine()

        de.last_stop = 3
        de.gf_low = 30
        de.gf_high = 85

        de.add_segment(68, 20)

        de.add_gas(0, 18, 45)
        de.add_gas(22, 50, 0)
        de.add_gas(6, 100, 0)

        info = de.calculate()

        self.assertEquals(12, len(info.stops))

        expected = list(range(36, 0, -3))
        self.assertEquals(expected, [s.depth for s in info.stops])

        expected = [2] * 3 + [3, 3, 2, 2, 3, 4, 8, 8, 14]
        self.assertEquals(expected, [s.time for s in info.stops])


    def test_u260_air(self):
        """
        Test U260 dive on air

        MultiDeco::

            Depth Stop Run Mix pO2 EAD
            Des 45 - 2 21 - -
            Lvl 45 22 25 21 1.15 45
            Asc 21 - 27 21 - -
            Stp 21 0:36 28 21 0.65 21
            Stp 18 1:00 29 21 0.59 18
            Stp 15 3:00 32 21 0.52 15
            Stp 12 3:00 35 21 0.46 12
            Stp 9 6:00 41 21 0.40 9
            Stp 6 8:00 49 21 0.34 6
            Stp 3 18 67 21 0.27 3
            Sfc - - 67 21 - -

            Dive# 1,  ZHLB + GF 30/85
            Elevation = 0 m
            CNS = 12%
            OTU's = 32
            Gas  21 = 4470 ltr.
            Decozone start = 32 m


        gfdeco::

                                      DECOMPRESSION CALCULATION PROGRAM
                                    Developed in FORTRAN by Erik C. Baker
             
             
             
            Description:    dive to 45 meters of seawater gauge (mswg) for 25 minutes on air      
             
            Gasmix Summary:                        FO2    FHe    FN2
                                      Gasmix # 1  0.210  0.000  0.790
             
             
                                                DIVE PROFILE
             
            Seg-  Segm.  Run   | Gasmix | Ascent    From     To      Rate    | Constant
            ment  Time   Time  |  Used  |   or     Depth   Depth    +Dn/-Up  |  Depth
              #   (min)  (min) |    #   | Descent  (mswg)  (mswg)  (msw/min) |  (mswg)
            ----- -----  ----- | ------ | -------  ------  ------  --------- | --------
              1     2.3    2.3 |    1   | Descent     0.     45.      20.0   |
              2    22.8   25.0 |    1   |                                    |    45.
             
             
                                           DECOMPRESSION PROFILE
             
                      Leading compartment enters the decompression zone at   30.9 mswg
                             Deepest possible decompression stop is   30.0 mswg
             
            Seg-  Segm.  Run   | Gasmix | Ascent   Ascent         |  DECO   STOP   RUN
            ment  Time   Time  |  Used  |   To      Rate    Grad. |  STOP   TIME   TIME
              #   (min)  (min) |    #   | (mswg) (msw/min) Factor | (mswg)  (min)  (min)
            ----- -----  ----- | ------ | ------ --------- ------ | ------  -----  -----
              3     2.4   27.4 |    1   |   21.    -10.0          |
              4     0.6   28.0 |    1   |                   0.30  |    21      1     28
              5     0.3   28.3 |    1   |   18.    -10.0          |
              6     0.7   29.0 |    1   |                   0.38  |    18      1     29
              7     0.3   29.3 |    1   |   15.    -10.0          |
              8     2.7   32.0 |    1   |                   0.46  |    15      3     32
              9     0.3   32.3 |    1   |   12.    -10.0          |
             10     2.7   35.0 |    1   |                   0.54  |    12      3     35
             11     0.3   35.3 |    1   |    9.    -10.0          |
             12     5.7   41.0 |    1   |                   0.61  |     9      6     41
             13     0.3   41.3 |    1   |    6.    -10.0          |
             14     7.7   49.0 |    1   |                   0.69  |     6      8     49
             15     0.3   49.3 |    1   |    3.    -10.0          |
             16    17.7   67.0 |    1   |                   0.77  |     3     18     67
             17     0.3   67.3 |    1   |    0.    -10.0          |
                                                            0.85  |

        """
        de = DecoEngine()

        de.last_stop = 3
        de.gf_low = 30
        de.gf_high = 85

        de.add_segment(45, 25)
        de.add_gas(0, 21)
        info = de.calculate()

        self.assertEquals(7, len(info.stops))

        expected = list(range(21, 0, -3))
        self.assertEquals(expected, [s.depth for s in info.stops])

        expected = [1, 1, 3, 3, 6, 10, 23]
        self.assertEquals(expected, [s.time for s in info.stops])


# vim: sw=4:et:ai
