#
# DecoMecha - diving decompression library.
#
# Copyright (C) 2013 by Artur Wroblewski <wrobell@pld-linux.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Gradient Factor change sanity unit tests.
"""

from decomecha import DecoEngine

from .util import TestCase


class GFTestCase(TestCase):
    """
    Gradient Factor sanity unit tests.
    """
    def test_oc_gf_deeper(self):
        """
        Test OC dive GF deeper stops
        """
        d1 = DecoEngine()
        d1.add_segment(42, 45)
        d1.add_gas(0, 27)
        d1.gf_low = 30
        d1.gf_high = 85
        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(42, 45)
        d2.add_gas(0, 27)
        d2.gf_low = 20
        d2.gf_high = 85
        i2 = d2.calculate()

        k1 = len(i1.stops)
        k2 = len(i2.stops)
        self.assertEquals(k1, k2 - 1)


    def test_oc_gf_longer(self):
        """
        Test OC dive GF longer stops
        """
        d1 = DecoEngine()
        d1.add_segment(42, 25)
        d1.add_gas(0, 27)
        d1.gf_low = 20
        d1.gf_high = 80
        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(42, 25)
        d2.add_gas(0, 27)
        d2.gf_low = 20
        d2.gf_high = 90
        i2 = d2.calculate()

        self._verify(i1, i2, 15)


    def test_ccr_gf_deeper(self):
        """
        Test CCR dive GF deeper stops
        """
        d1 = DecoEngine()
        d1.add_segment(52, 45)
        d1.add_gas(0, 27)
        d1.add_setpoint(0, 1.3)
        d1.gf_low = 30
        d1.gf_high = 85
        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(52, 45)
        d2.add_gas(0, 27)
        d2.add_setpoint(0, 1.3)
        d2.gf_low = 20
        d2.gf_high = 85
        i2 = d2.calculate()

        k1 = len(i1.stops)
        k2 = len(i2.stops)
        self.assertEquals(k1, k2 - 1)


    def test_ccr_gf_longer(self):
        """
        Test CCR dive GF longer stops
        """
        d1 = DecoEngine()
        d1.add_segment(42, 45)
        d1.add_gas(0, 27)
        d1.add_setpoint(0, 1.3)
        d1.gf_low = 20
        d1.gf_high = 80
        i1 = d1.calculate()

        d2 = DecoEngine()
        d2.add_segment(42, 45)
        d2.add_gas(0, 27)
        d2.add_setpoint(0, 1.3)
        d2.gf_low = 20
        d2.gf_high = 90
        i2 = d2.calculate()

        self._verify(i1, i2, 3)

# vim: sw=4:et:ai
