#
# DecoMecha - diving decompression library.
#
# Copyright (C) 2013 by Artur Wroblewski <wrobell@pld-linux.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Sanity unit tests utilities.
"""

import unittest
import logging

logger = logging.getLogger('decomecha.tests')

class TestCase(unittest.TestCase):
    """
    Sanity unit tests case base class.
    """
    def _verify(self, i1, i2, better_at, exclude={}):
        """
        Compare deco stops of two decompression dives.

        :Parameters:
         i1
            Decompression dive information with longer stops.
         i2
            Decompression dive information with shorter stops.
         better_at
            Depth at which deco stops should start to be definetely
            shorter.
        """        
        err_msg_tmpl = deco_cmp_error(i1, i2)

        for s1, s2 in zip(reversed(i1.stops), reversed(i2.stops)):
            err_msg = '\nErrors at depths: {}m {}m'.format(s1.depth, s2.depth) \
                + err_msg_tmpl

            if s1.depth in exclude or s2.depth in exclude:
                logger.warning('Depth is skipped in a test! Exclude list {}'
                        .format(tuple(sorted(exclude))))
                continue
            assert s1.depth == s2.depth, err_msg

            if s1.depth > better_at: # too deep for better gas to kick in
                self.assertTrue(s1.time >= s2.time, err_msg)
            else:
                self.assertTrue(s1.time > s2.time, err_msg)
        


def deco2str(info):
    return '\n'.join(' {:3}m {}min'.format(stop.depth, stop.time)
        for stop in info.stops)


def deco_cmp_error(i1, i2):
    return '\nFirst dive:\n{}\n\nSecond dive:\n{}'.format(deco2str(i1), deco2str(i2))


# vim: sw=4:et:ai
